unit FreeOTFEExplorerSettings;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Classes, // Required for TShortCut
  IniFiles,
  ComCtrls,
  CommonSettings;

{$IFDEF _NEVER_DEFINED}
// This is just a dummy const to fool dxGetText when extracting message
// information
// This const is never used; it's #ifdef'd out - SDUCRLF in the code refers to
// picks up SDUGeneral.SDUCRLF
const
  SDUCRLF = ''#13#10;
{$ENDIF}
  
const
  FREEOTFE_REGISTRY_SETTINGS_LOCATION = '\Software\FreeOTFEExplorer';
  
type
  TFreeOTFEExplorerSettings = class(TSettings)
  protected
    procedure _Load(iniFile: TCustomINIFile); override;
    function  _Save(iniFile: TCustomINIFile): boolean; override;

  public
    // General...
    OptDisplayToolbarVolume: boolean;
    OptDisplayToolbarVolumeLarge: boolean;
    OptDisplayToolbarVolumeCaptions: boolean;
    OptDisplayToolbarExplorer: boolean;
    OptDisplayToolbarExplorerLarge: boolean;
    OptDisplayToolbarExplorerCaptions: boolean;
    OptDisplayAddressBar: boolean;
    OptDisplayListViewStyle: TViewStyle;

    constructor Create(); override;
    destructor Destroy(); override;

    function  RegistryKey(): string; override;

  end;

var
  // Global variable
  Settings: TFreeOTFEExplorerSettings;

implementation

uses
  Windows,  // Required to get rid of compiler hint re DeleteFile
  SysUtils,  // Required for ChangeFileExt, DeleteFile
  Registry,
  Dialogs,
  SDUi18n,
  SDUGeneral,
  SDUDialogs,
  Menus,  // Required for ShortCutToText and TextToShortCut
  ShlObj;  // Required for CSIDL_PERSONAL

const
  SETTINGS_V1 = 1;
  SETTINGS_V2 = 2;

  // -- General section --
    OPT_DISPLAYTOOLBARVOLUME                     = 'DisplayToolbarVolume';
      DFLT_OPT_DISPLAYTOOLBARVOLUME                 = TRUE;
    OPT_DISPLAYTOOLBARVOLUMELARGE                = 'DisplayToolbarVolumeLarge';
      DFLT_OPT_DISPLAYTOOLBARVOLUMELARGE            = TRUE;
    OPT_DISPLAYTOOLBARVOLUMECAPTIONS             = 'DisplayToolbarVolumeCaptions';
      DFLT_OPT_DISPLAYTOOLBARVOLUMECAPTIONS         = TRUE;
    OPT_DISPLAYTOOLBAREXPLORER                   = 'DisplayToolbarExplorer';
      DFLT_OPT_DISPLAYTOOLBAREXPLORER               = TRUE;
    OPT_DISPLAYTOOLBAREXPLORERLARGE              = 'DisplayToolbarExplorerLarge';
      DFLT_OPT_DISPLAYTOOLBAREXPLORERLARGE          = TRUE;
    OPT_DISPLAYTOOLBAREXPLORERCAPTIONS           = 'DisplayToolbarExplorerCaptions';
      DFLT_OPT_DISPLAYTOOLBAREXPLORERCAPTIONS       = TRUE;  // Not the same as MS Windows Explorer, but makes
                                                             // the store/extract operations more obvious
    OPT_DISPLAYADDRESSBAR                        = 'DisplayAddressBar';
      DFLT_OPT_DISPLAYADDRESSBAR                    = TRUE;
    OPT_DISPLAYLISTVIEWSTYLE                     = 'DisplayListViewStyle';
      DFLT_OPT_DISPLAYLISTVIEWSTYLE                 = vsReport;


constructor TFreeOTFEExplorerSettings.Create();
begin
  inherited;

  Load();

end;

destructor TFreeOTFEExplorerSettings.Destroy();
begin

  inherited;
end;

procedure TFreeOTFEExplorerSettings._Load(iniFile: TCustomINIFile);
begin
  inherited _Load(iniFile);
  
  OptDisplayToolbarVolume           := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBARVOLUME,           DFLT_OPT_DISPLAYTOOLBARVOLUME);
  OptDisplayToolbarVolumeLarge      := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBARVOLUMELARGE,      DFLT_OPT_DISPLAYTOOLBARVOLUMELARGE);
  OptDisplayToolbarVolumeCaptions   := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBARVOLUMECAPTIONS,   DFLT_OPT_DISPLAYTOOLBARVOLUMECAPTIONS);
  OptDisplayToolbarExplorer         := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBAREXPLORER,         DFLT_OPT_DISPLAYTOOLBAREXPLORER);
  OptDisplayToolbarExplorerLarge    := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBAREXPLORERLARGE,    DFLT_OPT_DISPLAYTOOLBAREXPLORERLARGE);
  OptDisplayToolbarExplorerCaptions := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBAREXPLORERCAPTIONS, DFLT_OPT_DISPLAYTOOLBAREXPLORERCAPTIONS);
  OptDisplayAddressBar              := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYADDRESSBAR,              DFLT_OPT_DISPLAYADDRESSBAR);
  OptDisplayListViewStyle           := TViewStyle(iniFile.ReadInteger(SECTION_GENERAL,   OPT_DISPLAYLISTVIEWSTYLE,       ord(DFLT_OPT_DISPLAYLISTVIEWSTYLE)));

end;


function TFreeOTFEExplorerSettings._Save(iniFile: TCustomINIFile): boolean;
var
  allOK: boolean;
begin
  allOK := inherited _Save(iniFile);
  try
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBARVOLUME,            OptDisplayToolbarVolume);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBARVOLUMELARGE,       OptDisplayToolbarVolumeLarge);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBARVOLUMECAPTIONS,    OptDisplayToolbarVolumeCaptions);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBAREXPLORER,          OptDisplayToolbarExplorer);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBAREXPLORERLARGE,     OptDisplayToolbarExplorerLarge);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBAREXPLORERCAPTIONS,  OptDisplayToolbarExplorerCaptions);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYADDRESSBAR,               OptDisplayAddressBar);
    iniFile.WriteInteger(SECTION_GENERAL,     OPT_DISPLAYLISTVIEWSTYLE,            ord(OptDisplayListViewStyle));

  except
    on E:Exception do
      begin
      allOK := FALSE;
      end;
  end;

  Result := allOK;
end;

function TFreeOTFEExplorerSettings.RegistryKey(): string; 
begin
  Result := FREEOTFE_REGISTRY_SETTINGS_LOCATION;
end;

END.


