// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//



#include "main.h"
#include "FreeOTFEGUIlib.h"
#include "FreeOTFE4PDAGUIlib.h"
#include "FreeOTFE4PDAlib.h"
#include "FreeOTFE4PDAAPI.h"

#include "resource.h"

#include <aygshell.h>  // Required for SH... functions/definitions/etc
#pragma comment(lib, "aygshell") // Link in aygshell.lib

// Control IDs of the left/right WM5 menu buttons
#define TB_MENU_LEFT   101
#define TB_MENU_RIGHT  102

#define TB_BUTTON_IDX_LEFT   0  // 0 = lefthand button
#define TB_BUTTON_IDX_RIGHT  1  // 1 = righthand button

#define ID_SEPARATOR     0  // This line should be replaced with #include of 
                            // std header file with this definition

// =========================================================================
// Returns TRUE if the mountpoint specified is a valid mountpoint which may
// be used
BOOL ValidateMountpoint(
    HWND hWnd,
    WCHAR* mountpoint,
    BOOL SkipExistsCheck,
    BOOL silent
)
{
    BOOL retval = FALSE;

    if (wcslen(mountpoint) == 0)
        {
        if (!(silent))
            {
            MsgError(hWnd, TEXT("Please specify a mountpoint"));
            }
        }
    else if (wcscmp(RESERVED_MOUNTPOINT, mountpoint) == 0)
        {
        if (!(silent))
            {
            MsgError(hWnd, TEXT("The mountpoint specified is reserved; please enter a different mountpoint"));
            }
        }
    else if (wcspbrk(mountpoint, RESERVED_MOUNTPOINT_CHARS) != NULL)
        {
        if (!(silent))
            {
            MsgError(hWnd, TEXT("The mountpoint specified includes a invalid character; please enter a different mountpoint"));
            }
        }
    else if (
             (!(SkipExistsCheck)) &&
             (GetFileAttributes(mountpoint) != INVALID_FILE_ATTRIBUTES)
            )
        {
        if (!(silent))
            {
            MsgError(hWnd, TEXT("The mountpoint specified already exists; please enter a different mountpoint"));
            }
        }
    else
        {
        // Everything OK
        retval = TRUE;
        }

    return retval;
}


// =========================================================================
// Because we can't load a main menu with separators, any menuitem with
// an ID of ID_SEPARATOR will be replaced with a separator
void CopyMenu(HMENU hOrigMenu, HMENU hNewMenu)
{
    MENUITEMINFO menuItemInfo;
    int i;
    WCHAR buffer[1024];  // Just a value large enough to store menu text
    UINT newID;
    UINT newItemFlags;
    HMENU hNewPopup;

    i = 0;
    while (TRUE)
        {
        ZeroMemory(&menuItemInfo, sizeof(menuItemInfo));
        menuItemInfo.cbSize = sizeof(menuItemInfo);
        menuItemInfo.fMask  = (
                      MIIM_CHECKMARKS |
                      MIIM_DATA |
                      MIIM_ID |
                      MIIM_STATE |
                      MIIM_SUBMENU |
                      MIIM_TYPE
                     );
        menuItemInfo.dwTypeData = buffer;
        menuItemInfo.cch = sizeof(buffer);

        if (!(GetMenuItemInfo(hOrigMenu, i, TRUE, &menuItemInfo)))
            {
            break;
            }

        // Recursively copy submenus:
        newItemFlags = MF_BYPOSITION;
        newID = menuItemInfo.wID;
        if (
            (menuItemInfo.fType && MFT_SEPARATOR) ||
            (newID == ID_SEPARATOR)
           )
            {
            newItemFlags |= MF_SEPARATOR;
            }

        if (menuItemInfo.hSubMenu)
            {
            newItemFlags |= MF_POPUP;
            hNewPopup = CreatePopupMenu();
            CopyMenu(menuItemInfo.hSubMenu, hNewPopup);
            menuItemInfo.hSubMenu = hNewPopup;
            newID = (int)hNewPopup;
            }

        /*
        MF_STRING
        MF_UNCHECKED 
        MF_MENUBREAK 
        MF_GRAYED 
        MF_ENABLED 
        MF_CHECKED
        MF_MENUBARBREAK 
        MF_SEPARATOR 
        */

        // Add the copy...
        InsertMenu(hNewMenu, i, newItemFlags, newID, buffer);

        // Setup this for enabled and checkmarks...
        SetMenuItemInfo(hNewMenu, i, TRUE, &menuItemInfo);

        i++;
        }

} 


// =========================================================================
HWND SetupMenu_Simple(
    HWND hWnd, 
    int MenuID
)
{
    return SetupMenu_NonSoftkey(hWnd, MenuID);
}


// =========================================================================
HWND SetupMenu(
    HWND hWnd, 
    int MenuID, 
    int leftMenuItemCtrlID, 
    WCHAR* rightMenuTitle
)
{
    HWND retval;

    if (G_Options->SoftkeyMenus)
        {
        retval = SetupMenu_Softkey(
                                   hWnd, 
                                   MenuID, 
                                   leftMenuItemCtrlID, 
                                   rightMenuTitle
                                  );
        }
    else
        {
        retval = SetupMenu_NonSoftkey(hWnd, MenuID);
        }

    return retval;
}


// =========================================================================
// This function *requires* dummy menu resource IDR_MENU_DUMMY_LEFTRIGHT with
// two buttons
HWND SetupMenu_Softkey(
    HWND hWnd, 
    int MenuID, 
    int leftMenuItemCtrlID, 
    WCHAR* rightMenuTitle
)
{
    HWND retval;
    MENUITEMINFO menuInfoLeft;
    HMENU hMenuRight;
    TBBUTTON tbb[2];
    HMENU hMenuLoaded;
    WCHAR buffer[1024];  // Just a value large enough to store menu text
    WCHAR defaultRightTitle[1024];  // Just a value large enough to store menu text
    WCHAR* useRightTitle;

    retval = SetupMenu_NonSoftkey(hWnd, IDR_MENU_DUMMY_LEFTRIGHT);    

    useRightTitle = rightMenuTitle;
    if (useRightTitle == NULL)
        {
        wcscpy(defaultRightTitle, TEXT("Menu"));
        useRightTitle = defaultRightTitle;
        }

    if (retval != NULL) 
        {
        // Ditch the dummy buttons...
        SendMessage((HWND)retval, TB_DELETEBUTTON, 0, 0);
        SendMessage((HWND)retval, TB_DELETEBUTTON, 0, 0);

        hMenuLoaded = LoadMenu(G_hInstance, MAKEINTRESOURCE(MenuID)); 

        if (hMenuLoaded != NULL)
            {
            menuInfoLeft.cbSize = sizeof(menuInfoLeft);
            menuInfoLeft.fMask = (MIIM_TYPE | MIIM_ID);
            menuInfoLeft.cch = sizeof(buffer);
            menuInfoLeft.dwTypeData = buffer;

            // Identify menuitem for left button text
            if (GetMenuItemInfo(
                                  hMenuLoaded, 
                                  leftMenuItemCtrlID, 
                                  FALSE,
                                  &menuInfoLeft
                                 ))
                {
                // Lose any "..." following the command
                wcstok(buffer, TEXT("."));
                }
            else
                {
                wsprintf(buffer, TEXT("???"));
                }

            hMenuRight = CreatePopupMenu(); 
            CopyMenu(hMenuLoaded, hMenuRight);
            DestroyMenu(hMenuLoaded);

            memset(&tbb, 0, sizeof(tbb));

            // Setup left menubutton...
            tbb[0].idCommand = leftMenuItemCtrlID;
            tbb[0].iBitmap   = I_IMAGENONE;
            tbb[0].fsState   = TBSTATE_ENABLED;
            tbb[0].fsStyle   = (
                                TBSTYLE_BUTTON   |
                                TBSTYLE_AUTOSIZE |
                                0x80  // magic flag 
                               );
            tbb[0].dwData    = (DWORD)0;
            tbb[0].iString   = (int)buffer; 

            // Setup right menubutton...
            tbb[1].idCommand = TB_MENU_RIGHT;
            tbb[1].iBitmap   = I_IMAGENONE;
            tbb[1].fsState   = TBSTATE_ENABLED;
            tbb[1].fsStyle   = (
                                TBSTYLE_DROPDOWN |
                                TBSTYLE_BUTTON   |
                                TBSTYLE_AUTOSIZE |
                                0x80  // magic flag
                               );
            tbb[1].dwData    = (DWORD)hMenuRight;
            tbb[1].iString   = (int)useRightTitle; 

            SendMessage((HWND)retval, TB_ADDBUTTONSW, 2, (LPARAM)(&tbb));

            DrawMenuBar(retval); 
            }
        }

    return retval;

}


// =========================================================================
// Setup menu
// Returns: Menu HWND, or NULL on failure
HWND SetupMenu_NonSoftkey(HWND hWnd, int menuID)
{
    HWND retval = NULL;
    SHMENUBARINFO mbi;

    // Setup menu...
    memset(&mbi, 0, sizeof(mbi));

    mbi.cbSize     = sizeof(mbi);
    mbi.hwndParent = hWnd;

    if (menuID == IDR_MENU_NULL)
        {
        mbi.dwFlags = SHCMBF_EMPTYBAR;
        mbi.nToolBarId = 0;
        }
    else
        {
        // Needed if mbi.nToolBarId is a menu ID
        mbi.dwFlags = SHCMBF_HMENU;
        mbi.nToolBarId = menuID;
        }
                                       
    mbi.hInstRes   = G_hInstance;
    mbi.nBmpId     = 0;
    mbi.cBmpImages = 0; 

    if (!(SHCreateMenuBar(&mbi)))
        {
        MsgError(hWnd, TEXT("SHCreateMenuBar Failed"));
        }
    else
        {
        retval = mbi.hwndMB;        
        }

    return retval;
}


// =========================================================================
// Change the *left* softkey to the specified command/button text
// rightMenuTitle - If this is set to NULL, this function will try to use
//                  the text of newCmdID from the *right* softkey's menu
BOOL ChangeLeftSoftkey(
    HWND hCmdBar, 
    int newCmdID,
    WCHAR* rightMenuTitle
)
{
    BOOL allOK;
    TBBUTTONINFO buttonInfoLeft;
    TBBUTTON tbb;
    WCHAR existingMenuText[1024];  // Just a value large enough to store menu text
    WCHAR defaultRightTitle[1024];  // Just a value large enough to store menu text
    WCHAR* useRightTitle;
    UINT oldMenuCmdID;
    MENUITEMINFO existingMenuItemInfo;
    BOOL enableNewCtrl;

    // This function does nothing if we're not using softkey menus
    if (!(G_Options->SoftkeyMenus))
        {
        return TRUE;
        }

    allOK = TRUE;

    // Get the lefthand button
    if (allOK)
        {
        memset(&tbb, 0, sizeof(tbb));

        if (!(SendMessage(
                    hCmdBar,
                    TB_GETBUTTON,
                    (WPARAM)TB_BUTTON_IDX_LEFT,
                    (LPARAM)(&tbb)
                   )))
            {
            allOK = FALSE;
            }
        }


    // Locate the old button's command ID; needed to identify the button later
    if (allOK)
        {
        oldMenuCmdID = tbb.idCommand;

        memset(&buttonInfoLeft, 0, sizeof(buttonInfoLeft));

        buttonInfoLeft.cbSize = sizeof(buttonInfoLeft);
        buttonInfoLeft.dwMask = (
                               TBIF_COMMAND |
                               TBIF_STATE   | 
                               TBIF_LPARAM
                              );

        if (!(SendMessage(
                    hCmdBar,
                    TB_GETBUTTONINFO,
                    (WPARAM)oldMenuCmdID,
                    (LPARAM)(&buttonInfoLeft)
                   )))
            {
            allOK = FALSE;
            }
        }


    // Determine what text to give the replacement command item
    enableNewCtrl = TRUE;
    if (allOK)
        {
        // Use user supplied title...
        useRightTitle = rightMenuTitle;

        // Fallback to new command's title...
        if (useRightTitle == NULL)
            {
            // Attempt to get our righthand menu            
            memset(&tbb, 0, sizeof(tbb));
            if (SendMessage(
                        hCmdBar,
                        TB_GETBUTTON,
                        (WPARAM)TB_BUTTON_IDX_RIGHT,
                        (LPARAM)(&tbb)
                       ))
                {
                // Righthand menu obtained; get text of new menuitem
                existingMenuItemInfo.cbSize = sizeof(existingMenuItemInfo);
                existingMenuItemInfo.fMask = (
                                              MIIM_DATA    |
                                              MIIM_STATE   |
                                              MIIM_ID      |
                                              MIIM_SUBMENU |
                                              MIIM_TYPE
                                             );
                existingMenuItemInfo.dwTypeData = existingMenuText;
                existingMenuItemInfo.cch = sizeof(existingMenuText);
                
                if (GetMenuItemInfo(
                                    (HMENU)(tbb.dwData),
                                    newCmdID,
                                    FALSE,
                                    &existingMenuItemInfo
                                   ))
                    {
                    // Lose any "..." following the command
                    wcstok(existingMenuText, TEXT("."));

                    useRightTitle = existingMenuText;

                    if (existingMenuItemInfo.fState & MF_GRAYED)
                        {
                        enableNewCtrl = FALSE;
                        }
                    }

                }

            }

        // Fallback to "???"
        if (useRightTitle == NULL)
            {
            wcscpy(defaultRightTitle, TEXT("???"));
            useRightTitle = defaultRightTitle;
            }
        }


    // Update the button with the new text/command ID
    if (allOK) 
        {
        buttonInfoLeft.pszText = useRightTitle;
        buttonInfoLeft.cbSize = sizeof(buttonInfoLeft);
        buttonInfoLeft.idCommand = newCmdID;
        buttonInfoLeft.dwMask = (
                               TBIF_COMMAND |
                               TBIF_TEXT    |
                               TBIF_STATE   | 
                               TBIF_LPARAM
                              );

        if (!(SendMessage(
                          hCmdBar,
                          TB_SETBUTTONINFO,
                          (WPARAM)oldMenuCmdID,
                          (LPARAM)(&buttonInfoLeft)
                         )))
            {
            allOK = FALSE;
            }

        CommandBar_ItemEnable(hCmdBar, newCmdID, enableNewCtrl);
        }

    return allOK;
}


// =========================================================================
// =========================================================================

